"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeTag = exports.PreTag = exports.generateNewStylesheet = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const react_syntax_highlighter_1 = require("react-syntax-highlighter");
const react_syntax_highlighter_2 = __importDefault(require("react-syntax-highlighter"));
const prismStyles = __importStar(require("react-syntax-highlighter/dist/esm/styles/prism"));
const hljsStyles = __importStar(require("react-syntax-highlighter/dist/esm/styles/hljs"));
const react_native_gesture_handler_1 = require("react-native-gesture-handler");
const utils_1 = require("./utils");
const styes_1 = __importDefault(require("./styes"));
const styleCache = new Map();
const topLevelPropertiesToRemove = [
    "color",
    "textShadow",
    "textAlign",
    "whiteSpace",
    "wordSpacing",
    "wordBreak",
    "wordWrap",
    "lineHeight",
    "MozTabSize",
    "OTabSize",
    "tabSize",
    "WebkitHyphens",
    "MozHyphens",
    "msHyphens",
    "hyphens",
    "fontFamily",
];
function generateNewStylesheet({ stylesheet, highlighter, }) {
    if (styleCache.has(stylesheet)) {
        return styleCache.get(stylesheet);
    }
    stylesheet = Array.isArray(stylesheet) ? stylesheet[0] : stylesheet;
    const transformedStyle = Object.entries(stylesheet).reduce((newStylesheet, [className, style]) => {
        newStylesheet[className] = Object.entries(style).reduce((newStyle, [key, value]) => {
            if (key === "overflowX" || key === "overflow") {
                newStyle.overflow = value === "auto" ? "scroll" : value;
            }
            else if (value.includes("em")) {
                const [num] = value.split("em");
                newStyle[key] = Number(num) * 16;
            }
            else if (key === "background") {
                newStyle.backgroundColor = value;
            }
            else if (key === "display") {
                return newStyle;
            }
            else {
                newStyle[key] = value;
            }
            return newStyle;
        }, {});
        return newStylesheet;
    }, {});
    const topLevel = highlighter === "prism"
        ? transformedStyle['pre[class*="language-"]']
        : transformedStyle.hljs;
    const defaultColor = (topLevel && topLevel.color) || "#000";
    topLevelPropertiesToRemove.forEach((property) => {
        if (topLevel[property]) {
            delete topLevel[property];
        }
    });
    if (topLevel.backgroundColor === "none") {
        delete topLevel.backgroundColor;
    }
    const codeLevel = transformedStyle['code[class*="language-"]'];
    if (highlighter === "prism" && !!codeLevel) {
        topLevelPropertiesToRemove.forEach((property) => {
            if (codeLevel[property]) {
                delete codeLevel[property];
            }
        });
        if (codeLevel.backgroundColor === "none") {
            delete codeLevel.backgroundColor;
        }
    }
    styleCache.set(stylesheet, { transformedStyle, defaultColor });
    return { transformedStyle, defaultColor };
}
exports.generateNewStylesheet = generateNewStylesheet;
function createChildren({ stylesheet, fontSize, fontFamily, }) {
    let childrenCount = 0;
    return (children, defaultColor) => {
        childrenCount += 1;
        return children.map((child, i) => createNativeElement({
            node: child,
            stylesheet,
            key: `code-segment-${childrenCount}-${i}`,
            defaultColor,
            fontSize,
            fontFamily,
        }));
    };
}
function createNativeElement({ node, stylesheet, key, defaultColor, fontFamily, fontSize = 12, }) {
    const { properties, type, tagName: TagName, value } = node;
    const startingStyle = { fontFamily, fontSize, height: fontSize + 5 };
    if (type === "text") {
        return (<react_native_1.Text key={key} style={Object.assign({ color: defaultColor }, startingStyle)}>
        {value}
      </react_native_1.Text>);
    }
    else if (TagName) {
        const childrenCreator = createChildren({
            stylesheet,
            fontSize,
            fontFamily,
        });
        const style = utils_1.createStyleObject(properties.className, Object.assign({ color: defaultColor }, properties.style, startingStyle), stylesheet);
        const children = childrenCreator(node.children, style.color || defaultColor);
        return (<react_native_1.Text key={key} style={style}>
        {children}
      </react_native_1.Text>);
    }
}
function nativeRenderer({ defaultColor, fontFamily, fontSize, dark, startingLineNumber, }) {
    return ({ rows, stylesheet }) => {
        const width = Math.floor(Math.log10(rows.length)) * fontSize * 1.75;
        return rows.map((node, i) => {
            return (<react_native_1.View key={`row-wrapper${i + startingLineNumber}`} style={styes_1.default.row_container}>
          <react_native_1.View style={[
                styes_1.default.number_container,
                {
                    width: width,
                    backgroundColor: dark ? "#000" : "#fff",
                },
            ]}>
            <react_native_1.Text style={[
                styes_1.default.number,
                {
                    color: dark ? "#fff" : "#000",
                    fontSize: fontSize,
                },
            ]}>
              {`${i + startingLineNumber}`}
            </react_native_1.Text>
          </react_native_1.View>
          <react_native_1.View style={[
                styes_1.default.text_container,
                {
                    paddingLeft: width + 4,
                },
            ]}>
            <react_native_1.Text>
              {createNativeElement({
                node,
                stylesheet,
                key: `code-segment-${i}`,
                defaultColor,
                fontFamily,
                fontSize,
            })}
            </react_native_1.Text>
          </react_native_1.View>
        </react_native_1.View>);
        });
    };
}
exports.PreTag = (props) => {
    return (<react_native_gesture_handler_1.ScrollView {...props} horizontal={true} contentContainerStyle={styes_1.default.pretag}/>);
};
exports.CodeTag = (props) => {
    return (<react_native_gesture_handler_1.ScrollView {...props} horizontal={false} contentContainerStyle={styes_1.default.codetag}/>);
};
const fontFamilyDef = react_native_1.Platform.OS == "ios" ? "Menlo-Regular" : "monospace";
const fontSizeDef = 12;
const prismDefaultStyle = prismStyles.atomDark;
const defaultStyle = hljsStyles.agate;
const RNSyntaxHighlighter = (_a) => {
    var { fontFamily = fontFamilyDef, fontSize = fontSizeDef, children, dark = true, highlighter = "highlighterjs", style = highlighter === "prism" ? prismDefaultStyle : defaultStyle, startingLineNumber = 1 } = _a, rest = __rest(_a, ["fontFamily", "fontSize", "children", "dark", "highlighter", "style", "startingLineNumber"]);
    const { transformedStyle, defaultColor } = react_1.useMemo(() => {
        return generateNewStylesheet({
            stylesheet: style,
            highlighter,
        });
    }, [style, highlighter]);
    const Highlighter = react_1.useMemo(() => (highlighter === "prism" ? react_syntax_highlighter_1.Prism : react_syntax_highlighter_2.default), [highlighter]);
    const renderer = react_1.useMemo(() => {
        return nativeRenderer({
            defaultColor,
            fontFamily,
            fontSize,
            dark,
            startingLineNumber,
        });
    }, [defaultColor, fontFamily, fontSize, dark, startingLineNumber]);
    return (<Highlighter {...rest} PreTag={exports.PreTag} CodeTag={exports.CodeTag} style={transformedStyle} showLineNumbers={false} showInlineLineNumbers={false} wrapLines={true} horizontal={true} renderer={renderer}>
      {children}
    </Highlighter>);
};
exports.default = react_1.default.memo(RNSyntaxHighlighter);
//# sourceMappingURL=RNSyntaxHighlighter.js.map